package dpdknet

import (
	"context"
	"encoding/json"
	"net/http"

	"repository.basistech.ru/BASIS/decort-golang-sdk/internal/validators"
)

// GetRequest struct to get information about DPDK network
type GetRequest struct {
	// ID of the DPDK network
	// Required: true
	DPDKID uint64 `url:"dpdkId" json:"dpdkId" validate:"required"`
}

// Get DPDK network details as a RecordDPDKNet struct
func (d DPDKNet) Get(ctx context.Context, req GetRequest) (*RecordDPDKNet, error) {
	res, err := d.GetRaw(ctx, req)
	if err != nil {
		return nil, err
	}

	info := RecordDPDKNet{}

	err = json.Unmarshal(res, &info)
	if err != nil {
		return nil, err
	}

	return &info, nil
}

// GetRaw gets DPDK network details as an array of bytes
func (d DPDKNet) GetRaw(ctx context.Context, req GetRequest) ([]byte, error) {
	err := validators.ValidateRequest(req)
	if err != nil {
		return nil, validators.ValidationErrors(validators.GetErrors(err))
	}

	url := "/cloudbroker/dpdknet/get"

	res, err := d.client.DecortApiCall(ctx, http.MethodPost, url, req)
	return res, err
}
