package node

import (
	"context"
	"net/http"

	"repository.basistech.ru/BASIS/decort-golang-sdk/internal/validators"
)

// DecommissionRequest struct to set node status to DECOMMISSIONED
type DecommissionRequest struct {
	// Node ID
	// Required: true
	NID uint64 `url:"nid" json:"nid" validate:"required"`

	// Force node decommission
	// Default: false
	// Required: false
	Force bool `url:"force" json:"force"`
}

// Decommission sets node status to DECOMMISSIONED
func (n Node) Decommission(ctx context.Context, req DecommissionRequest) (string, error) {
	err := validators.ValidateRequest(req)
	if err != nil {
		return "", validators.ValidationErrors(validators.GetErrors(err))
	}

	url := "/cloudbroker/node/decommission"

	res, err := n.client.DecortApiCall(ctx, http.MethodPost, url, req)
	if err != nil {
		return "", err
	}

	return string(res), nil
}
