package sep

import (
	"context"
	"net/http"
	"strconv"

	"repository.basistech.ru/BASIS/decort-golang-sdk/internal/validators"
)

// DelPoolRequest struct to delete pool from storage endpoint (SEP)
type DelPoolRequest struct {
	// ID of SEP to delete pool
	// Required: true
	SEPID uint64 `url:"sep_id" json:"sep_id" validate:"required"`

	// Name of pool to delete
	// Required: true
	PoolName string `url:"pool_name" json:"pool_name" validate:"required"`
}

// DelPool deletes pool from SEP
func (s SEP) DelPool(ctx context.Context, req DelPoolRequest) (bool, error) {
	err := validators.ValidateRequest(req)
	if err != nil {
		return false, validators.ValidationErrors(validators.GetErrors(err))
	}

	url := "/cloudbroker/sep/delPool"

	res, err := s.client.DecortApiCall(ctx, http.MethodPost, url, req)
	if err != nil {
		return false, err
	}

	result, err := strconv.ParseBool(string(res))
	if err != nil {
		return false, err
	}

	return result, nil
}
